#ifndef VECTOR_H
#define VECTOR_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <gmp.h>

#define VECTOR_BLOCK_SIZE 16

typedef struct {
  int  dimension;
  int  blocks;
  int  nonzeros;
  int* i;
  mpq_t* value;
  int* ptr;
  mpq_t  tmp;
} vector;

typedef struct {
  int  dimension;
  int  blocks;
  int  nonzeros;
  int* i;
  double* value;
  int* ptr;
} vector_d;

void vector_init(vector* vec, int dimension);
vector* new_vector(int dimension);
void vector_free(vector** vec);
void vector_resize(vector* vec, int dimension);
void vector_copy(vector* to, vector* from);
void vector_zero_clear(vector* vec);
int  vector_element_is_zero(vector* vec, int i);
int  vector_get_nonzeros(vector* vec, int s, int t);
int  vector_get_first_nonzero_i(vector* vec);
int  vector_get_last_nonzero_i(vector* vec);
int  vector_get_nonzero_pos(vector* vec, int i);
void vector_rev_sgn(vector* v);
void vector_rev_element_sgn(vector* v, int i);
void vector_get_element(mpq_t* val, vector* vec, int i);
mpq_t* vector_get_element_ptr(vector* vec, int i);
void vector_set_element(vector* vec, mpq_t val, int i);
void vector_delete_element(vector* vec, int i);
void vector_add_element(vector* vec, mpq_t q, int i);
void vector_sub_element(vector* vec, mpq_t q, int i);
void vector_mul_element(vector* vec, mpq_t q, int i);
void vector_div_element(vector* vec, mpq_t q, int i);
void vector_neg_element(vector* vec, int i);
void vector_dec_dimension(vector* vec, int i);
void vector_scalar_product(vector* v, mpq_t s);
void vector_norm(mpq_t* val, vector* v);
void vector_inner_product(mpq_t* val, vector* a, vector* b);
void vector_swap_elements(vector* vec, int i1, int i2);
void vector_print(vector* vec);
int vector_check(vector* vec);

vector_d* new_vector_d(int dimension);
void vector_d_free(vector_d** v_d);
void vector_d_resize(vector_d* vec, int dimension);
int  vector_d_element_is_zero(vector_d* vec, int i);
int vector_d_get_nonzeros(vector_d* vec, int s, int t);
void vector_d_copy(vector_d* to, vector_d* from);
void vector_d_zero_clear(vector_d* vec);
void vector_get_d(vector* vec, vector_d* v_d);
double vector_d_get_element(vector_d* v_d, int i);
void vector_d_set_element(vector_d* vec, double val, int i);
void vector_d_delete_element(vector_d* vec, int i);
void vector_d_add_element(vector_d* vec, double d, int i);
void vector_d_sub_element(vector_d* vec, double d, int i);
void vector_d_mul_element(vector_d* v, double d, int i);
void vector_d_div_element(vector_d* v, double d, int i);
double vector_d_inner_product(vector_d* a, vector_d* b);
void vector_d_swap_elements(vector_d* vec, int i1, int i2);
void vector_d_dec_dimension(vector_d* v, int i);

#endif
