%{
open Proposition
open Atomic_formula
open Quantified_formula
open Parser_helper

let rec build_formula neutral f = function
  | [] -> neutral
  | [h] -> h
  | h::t -> f h (build_formula neutral f t);;
%}

%type <Atomic_formula.syntax_atomic Proposition.proposition *  Parser_helper.extent> proposition_eof
%type <Atomic_formula.syntax_atomic Proposition.proposition *  Parser_helper.extent> proposition
%type <Atomic_formula.linear_expression *  Parser_helper.extent> linear_expression

%type <(string, Atomic_formula.syntax_atomic) Quantified_formula.quantified_proposition * Parser_helper.extent> qproposition_eof
%type <(string, Atomic_formula.syntax_atomic) Quantified_formula.quantified_proposition * Parser_helper.extent> qproposition

%start proposition_eof
%start qproposition_eof

%token <string * Parser_helper.extent> Lex_identifier

%token <unit * Parser_helper.extent> Lex_lparen
%token <unit * Parser_helper.extent> Lex_rparen
%token <unit * Parser_helper.extent> Lex_lbracket
%token <unit * Parser_helper.extent> Lex_rbracket
%token <unit * Parser_helper.extent> Lex_lbrace
%token <unit * Parser_helper.extent> Lex_rbrace

%token <unit * Parser_helper.extent> Lex_exists
%token <unit * Parser_helper.extent> Lex_forall

%token <unit * Parser_helper.extent> Lex_and
%token <unit * Parser_helper.extent> Lex_or
%token <unit * Parser_helper.extent> Lex_comma
%token <unit * Parser_helper.extent> Lex_plus
%token <unit * Parser_helper.extent> Lex_minus
%token <unit * Parser_helper.extent> Lex_multiply
%token <unit * Parser_helper.extent> Lex_divide

%token <unit * Parser_helper.extent> Lex_kw_and
%token <unit * Parser_helper.extent> Lex_kw_or
%token <unit * Parser_helper.extent> Lex_kw_xor
%token <unit * Parser_helper.extent> Lex_kw_iff
%token <unit * Parser_helper.extent> Lex_kw_not

%token <Gmp.Z.t * Parser_helper.extent> Lex_integer

%token <unit * Parser_helper.extent> Lex_less
%token <unit * Parser_helper.extent> Lex_greater
%token <unit * Parser_helper.extent> Lex_less_equal
%token <unit * Parser_helper.extent> Lex_greater_equal
%token <unit * Parser_helper.extent> Lex_equal
%token <unit * Parser_helper.extent> Lex_not_equal

%token <unit * Parser_helper.extent> Lex_not
%token <unit * Parser_helper.extent> Lex_implies
%token <unit * Parser_helper.extent> Lex_true
%token <unit * Parser_helper.extent> Lex_false
%token <unit * Parser_helper.extent> Lex_EOF
%%

proposition_eof:
  proposition Lex_EOF { $1 }

qproposition_eof:
  qproposition Lex_EOF { $1 }

proposition:
  proposition1 { $1 }
| proposition1 Lex_implies proposition
    { Implies((fst $1), (fst $3)), (merge_extents $1 $3) }

proposition1:
  proposition2 { $1 }
| proposition1 Lex_or proposition2
    { Or((fst $1), (fst $3)), (merge_extents $1 $3) }

proposition2:
  proposition3 { $1 }
| proposition2 Lex_and proposition3
    { And((fst $1), (fst $3)), (merge_extents $1 $3) }

proposition3:
  proposition4 { $1 }
| Lex_not proposition4
    { Not(fst $2), (merge_extents $1 $2) }

proposition4:
  Lex_lparen proposition Lex_rparen { $2 }
| atomic { Atomic(fst $1), (snd $1) }

| Lex_kw_and Lex_lbracket proposition_list Lex_rbracket {
    (build_formula True (fun x y -> And(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_or Lex_lbracket proposition_list Lex_rbracket {
    (build_formula False (fun x y -> Or(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_xor Lex_lbracket proposition_list Lex_rbracket {
    (build_formula False (fun x y -> Xor(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_not Lex_lbracket proposition Lex_rbracket {
    Not(fst $3), (merge_extents $1 $4)
  }

proposition_list:
    /* */ { [] }
| proposition_nonempty_list { $1 }

proposition_nonempty_list:
  proposition Lex_comma proposition_nonempty_list { (fst $1)::$3 }
| proposition { [fst $1] }

identifier_list:
  Lex_identifier { [fst $1], (snd $1) }
| Lex_identifier Lex_comma identifier_list { (fst $1)::(fst $3), (merge_extents $1 $3) }

qproposition:
  qproposition1 { $1 }
| qproposition1 Lex_implies qproposition
    { Implies((fst $1), (fst $3)), (merge_extents $1 $3) }

qproposition1:
  qproposition2 { $1 }
| qproposition1 Lex_or qproposition2
    { Or((fst $1), (fst $3)), (merge_extents $1 $3) }

qproposition2:
  qproposition3 { $1 }
| qproposition2 Lex_and qproposition3
    { And((fst $1), (fst $3)), (merge_extents $1 $3) }

qproposition3:
  qproposition4 { $1 }
| Lex_not qproposition4
    { Not(fst $2), (merge_extents $1 $2) }

qproposition4:
| Lex_lparen qproposition Lex_rparen { $2 }
| qatomic { Atomic(QAtomic(fst $1)), (snd $1) }

| Lex_true { True, (snd $1) }
| Lex_false { False, (snd $1) }

| Lex_exists Lex_lbracket Lex_lbrace identifier_list
  Lex_rbrace Lex_comma qproposition Lex_rbracket {
    Atomic(Exists((fst $4), (fst $7))), (merge_extents $1 $8) }
| Lex_forall Lex_lbracket Lex_lbrace identifier_list
  Lex_rbrace Lex_comma qproposition Lex_rbracket {
    Atomic(Forall((fst $4), (fst $7))), (merge_extents $1 $8) }

| Lex_exists Lex_lbracket Lex_lbrace
  Lex_rbrace Lex_comma qproposition Lex_rbracket {
    Atomic(Exists([], (fst $6))), (merge_extents $1 $7) }
| Lex_forall Lex_lbracket Lex_lbrace
  Lex_rbrace Lex_comma qproposition Lex_rbracket {
    Atomic(Forall([], (fst $6))), (merge_extents $1 $7) }

| Lex_exists Lex_lbracket Lex_identifier
  Lex_comma qproposition Lex_rbracket {
    Atomic(Exists([fst $3], (fst $5))), (merge_extents $1 $6) }
| Lex_forall Lex_lbracket Lex_identifier
  Lex_comma qproposition Lex_rbracket {
    Atomic(Forall([fst $3], (fst $5))), (merge_extents $1 $6) }

| Lex_kw_and Lex_lbracket qproposition_list Lex_rbracket {
    (build_formula True (fun x y -> And(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_or Lex_lbracket qproposition_list Lex_rbracket {
    (build_formula False (fun x y -> Or(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_xor Lex_lbracket qproposition_list Lex_rbracket {
    (build_formula False (fun x y -> Xor(x, y)) $3), (merge_extents $1 $4)
  }
| Lex_kw_not Lex_lbracket qproposition Lex_rbracket {
    Not(fst $3), (merge_extents $1 $4)
  }

qproposition_list:
    /* */ { [] }
| qproposition_nonempty_list { $1 }

qproposition_nonempty_list:
  qproposition Lex_comma qproposition_nonempty_list { (fst $1)::$3 }
| qproposition { [fst $1] }


number:
  Lex_integer { (Gmp.Q.from_z (fst $1)), (snd $1) }
| Lex_integer Lex_divide Lex_integer { (Gmp.Q.div (Gmp.Q.from_z (fst $1)) (Gmp.Q.from_z (fst $3))), (merge_extents $1 $3) }
| Lex_lparen number Lex_rparen { (fst $2), (merge_extents $1 $3) }

linear_expression1:
| Lex_identifier { ([(fst $1), Gmp.Q.one], Gmp.Q.zero), (snd $1) }
| number linear_expression1 {
  (Atomic_formula.scale_expression (fst $1) (fst $2)), (merge_extents $1 $2) }
| number Lex_multiply linear_expression1 {
  (Atomic_formula.scale_expression (fst $1) (fst $3)), (merge_extents $1 $3) }
| number { ([], (fst $1)), (snd $1) }
| Lex_lparen linear_expression Lex_rparen {
    (fst $2), (merge_extents $1 $3) }

linear_expression:
| linear_expression1 { $1 }
| linear_expression Lex_plus linear_expression1 {
  (Atomic_formula.add_expressions (fst $1) (fst $3)), (merge_extents $1 $3) }
| linear_expression Lex_minus linear_expression1 {
  (Atomic_formula.sub_expressions (fst $1) (fst $3)), (merge_extents $1 $3) }
| Lex_minus linear_expression1 {
    (Atomic_formula.neg_expression (fst $2)), (merge_extents $1 $2) }

comparison_operator:
| Lex_equal { EQUAL, (snd $1) }
| Lex_not_equal { NOT_EQUAL, (snd $1) }
| Lex_less { LESS, (snd $1) }
| Lex_less_equal { LESS_EQ, (snd $1) }
| Lex_greater { GREATER, (snd $1) }
| Lex_greater_equal { GREATER_EQ, (snd $1) };;

atomic:
  Lex_identifier { Syntax_literal(fst $1), (snd $1) }
| linear_expression comparison_operator linear_expression {
    Syntax_comparison((fst $1), (fst $2), (fst $3)), (merge_extents $1 $3) }

qatomic:
  linear_expression comparison_operator linear_expression {
    Syntax_comparison((fst $1), (fst $2), (fst $3)), (merge_extents $1 $3) }
