#include "SimpSolver.h"
#include "ml_interface.h"

#define MODULE "Minisat"

#define CAMLCALL(id) extern "C" value ml_minisat_##id

static inline Solver*& solver_of_value(value r) {
  return *((Solver**) Data_custom_val(r));
}

static inline vec<Lit>*& clause_of_value(value r) {
  return *((vec<Lit>**) Data_custom_val(r));
}

extern "C" void ml_minisat_solver_finalize(value r) {
  // printf("finalizing solver\n");
  solver_of_value(r)->~Solver();
}

static struct custom_operations ml_minisat_solver_custom = {
  "Minisat.solver",
  &ml_minisat_solver_finalize
};

CAMLCALL(solver_create)(value dummy) {
  CAMLparam0();
  CAMLlocal1(r);
  r = caml_alloc_custom(&ml_minisat_solver_custom,
		   sizeof(Solver*),
		   0,
		   1);
  solver_of_value(r) = new SimpSolver;
  CAMLreturn(r);
}

CAMLCALL(solver_solve)(value solver) {
  CAMLparam1(solver);
  bool rv = solver_of_value(solver)->solve();
  CAMLreturn(Val_bool(rv));
}

CAMLCALL(solver_solve_with_assumptions)(value solver, value assumptions) {
  CAMLparam2(solver, assumptions);
  vec<Lit> assums_vec;
  while(assumptions != 1) {
    value pair = Field(assumptions, 0),
      var = Field(pair, 0), polarity = Field(pair, 1);
    assums_vec.push(Lit(Int_val(var), Bool_val(polarity)));
    assumptions = Field(assumptions, 1);
  }
  bool rv = solver_of_value(solver)->solve(assums_vec);
  CAMLreturn(Val_bool(rv));
}

CAMLCALL(solver_new_var)(value solver) {
  CAMLparam1(solver);
  Var v = solver_of_value(solver)->newVar();
  CAMLreturn(Val_int(v));
}

CAMLCALL(solver_add_clause)(value solver, value clause) {
  CAMLparam2(solver, clause);
  solver_of_value(solver)->addClause(*clause_of_value(clause));
  CAMLreturn(Val_unit);
}

static inline value value_of_lbool(lbool v) {
  switch (v.toInt()) {
  case 1: return Val_int(0);
  case -1: return Val_int(1);
  case 0: return Val_int(2);
  default: caml_failwith("bad lbool");
  }
}

CAMLCALL(solver_model_value)(value solver, value var, value polarity) {
  CAMLparam3(solver, var, polarity);
  lbool v =  solver_of_value(solver)->modelValue(Lit(Int_val(var), Bool_val(polarity)));
  CAMLreturn(value_of_lbool(v));
}

#if 0
CAMLCALL(solver_to_dimacs)(value solver, value filename) {
  CAMLparam2(solver, filename);
  solver_of_value(solver)->toDimacs(String_val(filename));
  CAMLreturn(Val_unit);
}
#endif

extern "C" void ml_minisat_clause_finalize(value r) {
  // printf("finalizing clause\n");
  clause_of_value(r)->~vec<Lit>();
}

static struct custom_operations ml_minisat_clause_custom = {
  "Minisat.clause",
  &ml_minisat_clause_finalize
};

CAMLCALL(clause_create)(value dummy) {
  CAMLparam0();
  CAMLlocal1(r);
  r = caml_alloc_custom(&ml_minisat_clause_custom,
		   sizeof(vec<Lit>*),
		   0,
		   1);
  clause_of_value(r) = new vec<Lit>;
  CAMLreturn(r);
}

CAMLCALL(clause_add_lit)(value clause, value var, value polarity) {
  CAMLparam3(clause, var, polarity);
  //  printf("adding var %d %d\n", Int_val(var), Bool_val(polarity));
  clause_of_value(clause)->push(Lit(Int_val(var), Bool_val(polarity)));
  CAMLreturn(Val_unit);
}
