(* let lexbuf = Lexing.from_string "x > 1 && y > 2 && (x + y <= 5 || x+ y <= 7)";;*)

let debug = false

let lexbuf = Lexing.from_channel stdin;; 

let (formula,_) = Formula_parser.proposition_eof Formula_lexer.normal_token lexbuf;;

if debug then
  begin
    print_endline "Parsed atomic:";
    Proposition.print Atomic_formula.print_syntax_atomic stdout formula;
    print_newline ()
  end;;

let formula_pr = Proposition.map_atomic Atomic_formula.compile_atomic formula;;

if debug then
  begin
    print_endline "Processed atomic:";
    Atomic_formula.print_pr_formula stdout formula_pr;
    print_newline ()
  end;;

let arguments =
  let l = ref [] in
  for i=1 to (Array.length Sys.argv)-1
  do
    l := Sys.argv.(i)::!l
  done;
  !l;;

if arguments = []
then
  begin
    match Smt_core.resolve_formula formula_pr with
    | Linear_system.UNSATISFIABLE _ ->
	print_endline "UNSATISFIABLE"
    | Linear_system.SATISFIABLE model ->
	print_endline "SATISFIABLE";
	print_endline "Booleans:";
	Hashtbl.iter (fun var_name v ->
	  Printf.printf "%s = %b\n" var_name v) model.Smt_core.boolean_model;
	print_endline "Rationals:";
	Hashtbl.iter (fun var_name v ->
	  Printf.printf "%s = %s\n" var_name (Gmp.Q.to_string v))
	  model.Smt_core.rational_model
  end
else
  begin
    List.iter
      (fun sys ->
	print_endline "disjunct";
	Linear_system.print_linear_system sys)
      (Smt_core.project_formula arguments formula_pr [])
   end;;

Gc.full_major ();;
