open Gmp;;

let rec zlinear_form_to_string f = function
  | [] -> "0"
  | [hv, hc] ->
      if Z.equal hc Z.one
      then f hv
      else (Z.to_string hc) ^ " " ^ (f hv)
  | (hv, hc)::t ->
      (if Z.equal hc Z.one
       then (zlinear_form_to_string f t) ^ " + " ^ (f hv)
       else
	 (if (Z.sgn hc) < 0
	  then
            (zlinear_form_to_string f t) ^ " - " ^
             (Z.to_string (Z.neg hc)) ^ " " ^ (f hv)
          else
	    (zlinear_form_to_string f t) ^ " + " ^
              (Z.to_string hc) ^ " " ^ (f hv)));;

let v = PPL.linear_expr_scale (Z.from_int 7) (PPL.linear_expr_from_variable 1);;
print_endline "v";;
PPL.linear_expr_println v;;

let w = PPL.linear_expr_add (PPL.linear_expr_from_scaled_variable (Z.from_int 5) 0) (PPL.linear_expr_from_z (Z.from_int 8));;
print_endline "w";;
PPL.linear_expr_println w;;

let p1 = PPL.nnc_poly_create 5 PPL.UNIVERSE;;
print_endline "p1";;
PPL.nnc_poly_println p1;;

let c1 = PPL.constraint_make v Linear_types.GREATER_EQ w;;
print_endline "c1";;
PPL.constraint_println c1;;

let c2 = PPL.constraint_make (PPL.linear_expr_from_scaled_variable (Z.from_int 3) 1) Linear_types.LESS_EQ (PPL.linear_expr_from_z (Z.from_int 2));;
print_endline "c2";;
PPL.constraint_println c2;;

let c3 = PPL.constraint_make (PPL.linear_expr_from_variable 0) Linear_types.GREATER_EQ (PPL.linear_expr_from_z (Z.from_int (-10)));;
print_endline "c3";;
PPL.constraint_println c3;;

let c4 = PPL.constraint_make (PPL.linear_expr_from_variable 0) Linear_types.GREATER_EQ (PPL.linear_expr_from_z (Z.from_int (-11)));;
print_endline "c4";;
PPL.constraint_println c4;;

PPL.nnc_poly_add_constraint p1 c1;;
PPL.nnc_poly_add_constraints p1 [c2; c3; c4];;
print_endline "p1";;
PPL.nnc_poly_println p1;;

PPL.nnc_poly_remove_higher_space_dimensions p1 1;;
print_endline "projected p1";;
PPL.nnc_poly_println p1;;

print_endline "projected p1 constraint list";;
List.iter (fun ct -> Linear_types.print_izlconstraint (PPL.constraint_read ct))
  (PPL.nnc_poly_minimized_constraints p1);;

let lpp = PPL.lp_problem_create [c1; c2; c3; c4]
  (PPL.linear_expr_from_variable 0) Linear_types.MAXIMIZATION;;

match PPL.lp_problem_solve lpp with
  | PPL.UNFEASIBLE_LP_PROBLEM -> print_endline "unfeasible"
  | PPL.UNBOUNDED_LP_PROBLEM -> print_endline "unbounded"
  | PPL.OPTIMIZED_LP_PROBLEM ->
      let (num, den) = PPL.lp_problem_optimal_value lpp
      and gen = PPL.lp_problem_optimizing_point lpp in
        Printf.printf "optimized = %s/%s (%b: %s/%s)\n" (Z.to_string num) (Z.to_string den) (PPL.generator_is_point gen) (zlinear_form_to_string (fun d -> Printf.sprintf "v%d" d) (PPL.generator_read_coefficients gen)) (Z.to_string (PPL.generator_read_divisor gen));
        PPL.generator_println gen;;
